#include "mikmod.h"
#include "forms.h"
#include "engine.h"
#include "main_panel.h"
#include "playlist.h"
#include "setup_panel.h"
#include "xpm.h"
#include "play.h"

engine_status estatus={NULL, "", 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, ""};

int is_playing(void)
{
	return estatus.playing;
}

void update_pos(int pos)
{
	if ( MP_Ready() ) 
	{
		if ( skip_next() ) stop();
	} else MD_Update();
	if ( estatus.scroll )
		if ( estatus.scroll_count == 0 )
		{
			if ( panel->title_pos == estatus.titlesize ) panel->title_pos = 0;
			else panel->title_pos++;
			printf("Scrolling...\n");
			update_display();
		} else estatus.scroll_count--;
	if ( estatus.pattern != pos ) 
	{
		estatus.pattern = pos;	
		update_display();
	}
}

void setup_engine(void)
{
	md_mixfreq = config->frequency;
	md_dmabufsize = 32768;
	md_device = 0;
	md_mode = 0;
	if ( config->flags & PF_STEREO ) md_mode |= DMODE_STEREO;
	if ( config->flags & PF_16BITS ) md_mode |= DMODE_16BITS;
	if ( config->flags & PF_INTERPOLATION ) md_mode |= DMODE_INTERP;
	MD_RegisterPlayer(tickhandler);
	if ( !MD_Init() ) printf("Cannot setup driver to the required parameters\n");
	config->flags &= 8;
	if ( md_mode & DMODE_STEREO ) config->flags |= PF_STEREO;
	if ( md_mode & DMODE_16BITS ) config->flags |= PF_16BITS;
	if ( md_mode & DMODE_INTERP ) config->flags |= PF_INTERPOLATION;
	config->frequency = md_mixfreq;
} 


void refresh(void)
{
	int	i;
	
	if ( setup->updated )
	{
		if ( estatus.playing ) stop();
		MD_Exit();
		setup_engine();
	}
	estatus.maxline = fl_get_browser_maxline(list->list_browser);
	if ( estatus.maxline )
	{
		if ( estatus.loaded )
		{
			for ( i=1 ; i<=estatus.maxline ; i++)
				if ( strcmp(fl_get_browser_line(list->list_browser, i),
					estatus.path) == 0 )
				{
					estatus.index = i;
					break;
				}
			if ( i > estatus.maxline ) 
			{
				estatus.index = 1;
				stop();
				free_module();
				estatus.maxpos = 0;
				estatus.loaded = 0;
			}
		} else 
		{
			estatus.index = 1;
			estatus.maxpos = 0;
			estatus.pattern = 0;
		}
	} else 
	{
		if ( estatus.loaded ) free_module();
		estatus.index = 0;
		estatus.maxpos = 0;
		estatus.pattern = 0;
	}	
	update_display();
}

void update_display(void)
{
	int	i,j, r;
	char	tbuf[2][21];
	
	if ( estatus.maxline == 0 )
		strncpy(tbuf[0], "No modules in list ", 20);
	else
		if ( estatus.loaded )
			if ( estatus.scroll )
			{
				strncpy(tbuf[0], &estatus.title[panel->title_pos], 20);
				if ( (r = estatus.titlesize - panel->title_pos) < 20 )
					strncat(tbuf[0], estatus.title, r);
			} else strncpy(tbuf[0], estatus.title, 20);
		else strncpy(tbuf[0], "No module loaded", 20);
	sprintf(tbuf[1], "Song:%02d/%02d Pos:%02d/%02d", estatus.index, 
			estatus.maxline, estatus.pattern, estatus.maxpos);
	fl_freeze_form(panel->main_panel);
	for ( i=0 ; i<2 ; i++ )
		for ( j=0; j<20 ; j++ )
			if ( tbuf[i][j] != panel->led_display_buffer[i][j] )
			{
				panel->led_display_buffer[i][j] = tbuf[i][j];
				fl_set_pixmap_data(panel->led_display[i][j], char2pixmap(tbuf[i][j]));
			}
	fl_unfreeze_form(panel->main_panel);
}

int load_module(int i)
{
	char * buf;

	buf = fl_get_browser_line(list->list_browser, i);
	if ( (estatus.module = ML_LoadFN(buf)) == NULL )
	{
		printf("Error loading module %s\n", buf);
		return 1;
	}
	estatus.index = i;
	estatus.pattern = 0;
	estatus.loaded = 1;
	estatus.maxpos = estatus.module->numpos - 1;
	strcpy(estatus.path, buf);
	strncpy(estatus.title, estatus.module->songname, 64);
	estatus.titlesize = strlen(estatus.title);
	estatus.scroll = 0; /*(estatus.titlesize > 20) ;*/
	estatus.scroll_count = SCROLL_CNT;
	panel->title_pos = 0;
	return 0;
}

int free_module(void)
{
	if ( estatus.playing ) stop();
	if ( estatus.module != NULL )
	{
		ML_Free(estatus.module);
		estatus.loaded = 0;
		estatus.module = NULL;
		estatus.maxpos = 0;
		estatus.pattern = 0;
		return 0;
	} else return 1;
}

void tickhandler(void)
{
	MP_HandleTick();
	MD_SetBPM(mp_bpm);
}

int play(void)
{
	if ( estatus.playing ) return 1;
	if ( !estatus.loaded )
		if ( load_module(estatus.index) ) return 1;
	estatus.playing = 1;
	fl_set_pixmapbutton_data(panel->stop_button, stop_off_xpm);
	fl_set_pixmapbutton_data(panel->play_button, play_on_xpm);
	update_display();
	panel->status |= SF_PLAY;
	MP_Init(estatus.module);
	md_numchn = estatus.module->numchn;
	if ( estatus.pattern != 0 ) MP_SetPosition(estatus.pattern);
	MD_PlayStart();
	return 0;
}

int stop(void)
{
	if ( !estatus.playing ) return 1;
	estatus.playing = 0;
	fl_set_pixmapbutton_data(panel->play_button, play_off_xpm);
	fl_set_pixmapbutton_data(panel->stop_button, stop_on_xpm);
	panel->status |= SF_STOP;
	MD_PlayStop();
	return 0;
}

int skip_prev(void)
{
	int was_playing;

	if ( estatus.index < 2 ) return 1;
	was_playing = estatus.playing;
	estatus.index--;
	if ( estatus.loaded )
	{
		free_module();
		load_module(estatus.index);
	}
	if ( was_playing ) play();
	else update_display();
	return 0;
}

int skip_next(void)
{
	int was_playing;

	if ( estatus.index >= estatus.maxline ) return 1;
	was_playing = estatus.playing;
	estatus.index++;
	if ( estatus.loaded )
	{
		free_module();
		load_module(estatus.index);
	}
	if ( was_playing ) play();
	else update_display();
	return 0;
}

int ff(void)
{
	if ( estatus.pattern < estatus.maxpos )
	{
		estatus.pattern++;
		update_display();
		if ( estatus.playing ) MP_NextPosition();
		return 0;
	} else return 1;
}

int rew(void)
{
	if  ( estatus.pattern > 0 )
	{	
		estatus.pattern--;
		update_display();
		if ( estatus.playing ) MP_PrevPosition();
		return 0;
	} else return 1;
}